<?php
require_once('libs/constants.php');
require_once('libs/validators.php');
require_once('config/myva.php');

/**
 * AppController
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Controllers
 */
class AppController extends Controller {

  var $name = 'AppController';
  /** 
   * Actionit, jotka eivt vaadi kirjautumista.
   * Aliluokkien on halutessaan peitettv tm muuttuja konstruktorissaan.
   */
  protected $need_not_login = array();
  /**
   * Actionit, jotka vaativat admin-oikeudet.
   * Aliluokkien on halutessaan peitettv tm muuttuja konstruktorissaan.
   */
  protected $need_admin = array();
  /** 
   * Actionit, jotka tarvitsevat valikot ym. nkymn.
   * Aliluokkien on halutessaan peitettv tm muuttuja konstruktorissaan.
   */
  protected $need_menus = array();
  /*
   * Onko kutsuva selain Internet Explorer.
   * Alustetaan konstruktorissa ja asetetaan nkymlle beforeFilter:ss.
   */
  protected $is_ie;
  
  protected $sanitizer;

  function __construct (){
    parent::__construct();
    uses('sanitize');
    $this->sanitizer = new Sanitize();
    $this->is_ie = (strpos($_SERVER['HTTP_USER_AGENT'], ' MSIE ') !== false);
  }

  /**
   * BeforeFilter ajetaan ennen jokaista AppController-luokan toteuttavaa metodikutsua.
   * Metodia on voitava kutsua mys ksin, joten metodin on toimittava vaikka sit kutsuttaisiin useita kertoja.
   */
  function beforeFilter(){
    /*
     * Vaihdetaan kontrollerin nimi alkavaksi pienell kirjaimella.
     * Se korjaa ongelman, ett kontrollerin nimest riippuvia tiedostoja ei muuten osata ladata oikein.
     * Huom, ett params['url'] j korjaamatta!
     * Tten kontrollereita saisi kutsua vain pienell kirjaimella alkavalla nimell.
     */
    $this->params['controller'] = substr_replace($this->params['controller'], strtolower(substr($this->params['controller'], 0, 1)), 0, 1);

    if ($this->loginRequests()) {
      $this->checkLogin();
    }
    if (in_array($this->params['action'], $this->need_menus)) {
      $this->set('campaign_menu',
      $this->requestAction('/campaignsUsers/getUserCampaigns/' . urlencode($this->params['url']['url']),
        array('return')));
      $this->set('page_menu',
      $this->requestAction('/campaignsUsers/getMenu/' . urlencode($this->params['url']['url']),
        array('return')));
      $this->set('info_bar', '&nbsp;');
      $this->set('content_class', 'page_content');
    } else {
      $this->set('content_class', 'popup_content');
    }
    $this->set('is_ie', $this->is_ie);
  }

  /**
   * Tarkistaa kirjautumisen, vanhentuneen kirjautumisen tai kirjautumisen puuttumisen tapauksessa
   * ohjaa etusivulle
   */
  private function checkLogin(){
    // Onko sessio tyhj
    if ($this->Session->check('user')) {
      $user = $this->Session->read('user');
    }
    if (isset($user)
    && ($user['campaign_admin'] || ! in_array($this->params['action'], $this->need_admin))) {
      
      if (time() - $this->Session->read('last_user_event_time') > MYVA_CONFIG::SESSION_TIMEOUT){
        $this->Session->delete($user);
        $this->flash("Kirjautuminen vanhentunut", "/users/index");
      } else {
        //Annetaan lisaikaa
        
        $this->Session->write('last_user_event_time', time());
        
        
      }
    } else {
      $this->redirect('/users/index');
      exit();
    }
  }


  /**
   * Palauttaa tiedon siit, tehdnk login vai ei
   */
  private function loginRequests(){
    return (! in_array($this->params['action'], $this->need_not_login));
  }

  /**
   * Palauttaa tiedon, onko annettu sivun tunnistetieto kelvollinen
   *
   * @param $page_id Tarkistettava sivun tunnistetieto.
   * @return true jos kelvollinen, muutoin false.
   */
  protected function isValidPageId($page_id) {
    foreach (array_keys(MYVA_CONST::$MENU_PAGES) as $valid_id) {
      if ($page_id === $valid_id) {
        return true;
      }
    }
    return false;
  }

  /**
   * Tulostaa parametrin <pre>-tagien sisn, jos DEBUG > 0
   */
  public function printVariable($variable){
    if (DEBUG > 0) {
      echo "<pre>";
      print_r($variable);
      echo "</pre>";
    }
  }

  /**
   * Asettaa inforivin tekstin ja lykk tarvittaessa mahdollisen kannustusviestin nkymist.
   *
   * @param string $text Inforiville asetettava teksti.
   * @param string $class Inforiville asetettavan tekstin class-attribuutin arvo.
   *    * 
   * @return boolean Totuusarvo, syrjytettiink kannustusviesti.
   */
  protected function setInfoBar($text, $class = '') {
    $this->set('info_bar', '<span class="' . $class . '">' . $text . '</span>');
    return false;
  }
  
  
  /**
   * Suorittaa rekursiivisesti parametrina annetun funktion kaikille taulukon string-tyyppisille alkioille.
   * Muun tyyppisiin alkioihin ei kosketa.
   * Metodi ei palauta mitn, vaan muokkaa suoraan viiteparametrina saamaansa taulukkoa.
   * 
   * @param array &$array Ksiteltv taulukko.
   * @param string $function String-alkioille suoritettava funktio.
   * @param class $class Luokka, jonka metodi funktio on. Jos luokkaa ei ole, annetaan arvoksi null.
   */
  protected function forStringsInArray(&$array, $function, $class = null) {
    $class_empty = empty($class);
    foreach(array_keys($array) as $key) {
      if (is_string($array[$key])) {
        if ($class_empty) {
          $array[$key] = $function($array[$key]);
        } else {
          $array[$key] = $class->$function($array[$key]);
        }
      } else if (is_array($array[$key])) {
        $this->forStringsInArray($array[$key], $function, $class);
      }
    }
  }

}
?>
