<?php

/**
 * AppModel
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Models
 */
class AppModel extends Model{
  const DB_DATE_FORMAT = 'Y-m-d';
  const DB_TIME_FORMAT = 'H:i:s';
  
  /**
   * Palauttaa campaigns_users taulusta id-kentn
   * @param int $user_id
   * @param int $campaign_id
   * @return int, id tai null, jos id:t ei ole
   */
  public function getCampaignsUserId($user_id, $campaign_id){

    $tulos = $this->query("Select id from campaigns_users where user_id = " . $user_id . " and campaign_id = " . $campaign_id);
    // Jos tulee tyhj setti, niin null
    if (empty($tulos)){
      return null;
    }

    $campaigns_user_id = $tulos[0]['campaigns_users']['id'];
    return $campaigns_user_id;
  }
  

  /**
   * Muotoilee merkkijonosytteet tietokannalle sopiviksi.
   * 
   * @param string $str Muotoilematon merkkijono.
   * 
   * @return string Muotoiltu merkkijono.
   * @deprecated Kontrollereissa kytetn CakePHP:n $this->sanitize->sql().
   */
  protected function escapeString($str) {
  	$funct = MYVA_CONFIG::ESCAPE_FUNCTION;
    return $funct($str);
  }

  /**
   * Muotoilee parametrina saamansa ajan toisena parametrina saamaansa muotoon.
   * @param $date_string UNIX timestamp tai strtotime()-merkkijono. Jos tyhj, palautetaan null.
   * @param $format_string date()-funktion muotoilumerkkijono.
   * @return null, jos $date_string on tyhj merkkijono. false, jos $date_string on virheellinen.
   *         Muutoin palautetaan aika pyydetyss muodossa.
   */
  protected function formatTime($date_string, $format_string) {
    if (empty($date_string)) {
      return null;
    } else if (is_integer($date_string) || is_numeric($date_string)) {
      $unix_timestamp = intval($date_string);
    } else {
      $unix_timestamp = strtotime($date_string);
    }
    if ($unix_timestamp < 0) {
      return false;
    }
    return (date($format_string, $unix_timestamp));
  }

  /**
   * Muotoilee parametrina saamansa ajan tietokannan pivmrmuotoon vvvv-kk-pp.
   * @param $date_string UNIX timestamp tai strtotime()-merkkijono. Jos tyhj, palautetaan null.
   * @return string Aika pivmrmuodossa vvvv-kk-pp.
   */
  public function timeToDbDate($date_string) {
    return $this->formatTime($date_string, AppModel::DB_DATE_FORMAT);
  }


  /**
   * Muotoilee parametrina saamansa ajan tietokannan aikamuotoon tt:mm:ss.
   * @param $date_string UNIX timestamp tai strtotime()-merkkijono. Jos tyhj, palautetaan null.
   * @return string Aika muodossa tt:mm:ss.
   */
  public function timeToDbTime($date_string) {
    return $this->formatTime($date_string, AppModel::DB_TIME_FORMAT);
  }


  /**
   * Muotoilee parametrina saamansa ajan tietokannan pivmrmuotoon vvvv-kk-pp tt:mm:ss.
   * @param $date_string UNIX timestamp tai strtotime()-merkkijono. Jos tyhj, palautetaan null.
   * @return string Aika pivmrmuodossa vvvv-kk-pp tt:mm:ss.
   */
  public function timeToDbDateTime($date_string) {
    return $this->formatTime($date_string, AppModel::DB_DATE_FORMAT . ' ' . AppModel::DB_TIME_FORMAT);
  }
  
  
  /**
   * Palauttaa totuusarvon, onko parametrina annettu merkkijono validi db-muotoinen aika.
   * Tarkistaa pivn kuukauden mukaan, huomioi karkausvuoden.
   * 
   * @param DateString $date_string Db-muotoinen aikamerkkijono.
   */
  protected function isValidDbTime($date_string){
    
    $leapyear = false;
    $time = explode(" " , $date_string);
    
    // Pit olla kaksi osaa
    if (!count($time) == 2){
      
      return false;
    }
    // Meill on siis kaksi osaa
	// tsekataanpas eka
	$date = explode("-" ,$time[0]);
	
	if (!count($date) == 3 ){
	  
	  return false;
	} else {
	  // Tarkistetaan vuosi
	  if (!is_numeric($date[0]) || $date[0] <0){
	    
	    return false; 
	  }
	  //Mritetn karkausvuosi
	  if ($date[1]%4 == 0 && $date[1]%100 <> 0){
	    $leapyear == true;	  
	  }  
	  // Tarkistetaan kuukausi
	  if (!is_numeric($date[1]) || $date[1] < 1 || $date[1] > 12){
	    
	    return false; 
	  }
	  // Sitten pivmr
	  
	  if ($date[2] < 1 || $date[2] > 31 ||
	        ($date[2] == 31 && $date[1] < 6 && $date[1] % 2 == 1 )
	        || ($date[2] == 31 && $date[1] > 6 && $date[1] % 2 == 0 )
	        || (!$leapyear && (int)$date[1] == 2 && $date[2] > 28)
	        || ($leapyear && (int)$date[1] == 2 && $date[2] > 29)){
	  
	          return false;
	        }
	  
	}
    // Tarkistaan aika
	$timeportion = explode (":", $time[1]);
	if (!count($timeportion) == 3 || $timeportion[0] < 0 || $timeportion[0] > 24 || $timeportion[1] <0 || $timeportion[1] >59 || $timeportion[2] <0 || $timeportion[1] >59){
	  return false;
	}
	return true;
  }

}
?>
