<?php
/**
 * MYVA validointivakiot.
 * 
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Definitions
 */
 


require_once(CAKE . '/libs/validators.php');


/**
 * Tekee parametrina annetusta snnllisen lausekkeen sisltvst vakiosta vakion nimelt
 * <VAKIO>_OR_EMPTY, joka hyvksyy vakion sisltmn snnllisen lausekkeen lisksi mys tyhjn merkkijonon.
 * 
 * @param $valid Validointivakion nimi
 */
function validOrEmpty($valid) {
  define($valid . '_OR_EMPTY', preg_replace('/^\/(.*[^\/])\/([a-zA-Z]*)$/', '/(^$)|($1)/$2', constant($valid)));
}


/**
 * CakePHP:n validointivakioita tydennettyn tyhjn sallivalla ehdolla.
 */
validOrEmpty('VALID_NUMBER');
validOrEmpty('VALID_EMAIL');
validOrEmpty('VALID_YEAR');

/**
 * Validi kyttjtunnus.
 */
define('VALID_USERNAME', '/^[a-z0-9]{3,16}$/');

/**
 * Validi salasana
 * TODO: vaatimus erityyppisist merkeist
 */
define('VALID_PASSWORD', '/^.{8,}$/');

/**
 * Validi SHA1-tiiviste.
 */
define('VALID_SHA1', '/^[0-9a-f]{40}$/');

/**
 * Validi puhelinnumero.
 * TODO: tarkennettava
 */
define('VALID_PHONE_NUMBER', '/^(\\+|(\([0-9]+\)))?[0-9\\- ]{3,}$/');
validOrEmpty('VALID_PHONE_NUMBER');

/**
 * Validi tyhj merkkijono
 */
define('VALID_EMPTY', '/^$/');

/**
 * Validi URL-osoite
 * Otettu osoitteesta: http://regexlib.com/REDetails.aspx?regexp_id=1374
 * Kauttaviivat (/) korjattu muotoon \/ ja lauseke muutettu testaamaan koko merkkijonoa (^$)
 */
define('VALID_URL', '/^(([\w]+:)?\/\/)?(([\d\w]|%[a-fA-f\d]{2,2})+(:([\d\w]|%[a-fA-f\d]{2,2})+)?@)?([\d\w][-\d\w]{0,253}[\d\w]\.)+[\w]{2,4}(:[\d]+)?(\/([-+_~.\d\w]|%[a-fA-f\d]{2,2})*)*(\?(&?([-+_~.\d\w]|%[a-fA-f\d]{2,2})=?)*)?(#([-+_~.\d\w]|%[a-fA-f\d]{2,2})*)?$/');
validOrEmpty('VALID_URL');

/**
 * Validi Y-tunnus
 */
define('VALID_YID', '/^[0-9]{1,14}-[0-9]$/');
validOrEmpty('VALID_YID');


/**
 * Validi pivmr ja/tai kellonaika suomalaisittain (pp.kk.vvvv tt.mm.ss).
 */
define('VALID_DATETIME',
  '/^(((0?[1-9])|([1-2][0-9])|(3[0-1]))\.((0?[1-9])|(1[0-2]))\.([1-2][0-9]{3})( +(([0-1]?[0-9])|(2[0-4]))((\.|:)([0-5]?[0-9])){1,2})?)|((((0?[1-9])|([1-2][0-9])|(3[0-1]))\.((0?[1-9])|(1[0-2]))\.([1-2][0-9]{3}) +)?(([0-1]?[0-9])|(2[0-4]))((\.|:)([0-5]?[0-9])){1,2})$/');
validOrEmpty('VALID_DATETIME');


/**
 * Validi pivmr ja/tai kellonaika tietokantamuodossa (vvvv-kk-pp tt:mm:ss).
 */
define('VALID_DB_DATETIME',
  '/^([1-2][0-9]{3}-((0[1-9])|(1[0-2]))-((0[1-9])|([1-2][0-9])|(3[0-1]))( +(([0-1][0-9])|(2[0-4]))(:([0-5][0-9])){1,2})?)|(([1-2][0-9]{3}-((0[1-9])|(1[0-2]))-((0[1-9])|([1-2][0-9])|(3[0-1])) +)?(([0-1][0-9])|(2[0-4]))(:([0-5][0-9])){1,2})$/');
validOrEmpty('VALID_DB_DATETIME');


/**
 * Validi kokonaisluku. Voi olla negatiivinen tai positiivinen.
 */
define('VALID_INTEGER', '/^[+-]?\\d+$/');
validOrEmpty('VALID_INTEGER');

/**
 * Validi ei-negatiivinen kokonaisluku. Voi sislt etumerkin (+).
 */
define('VALID_NON_NEG_INT', '/^[+]?\\d+$/');
validOrEmpty('VALID_NON_NEG_INT');

/**
 * Validi boolean-arvo. Hyvksyy 0, 1, true, false
 */
define('VALID_BOOLEAN', '/^((0)|(1)|(true)|(false))$/');
validOrEmpty('VALID_BOOLEAN');

?>
