<?php
/**
 * Campaign
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos 
 * @version
 * @package myva
 * @subpackage Models
 */
class Campaign extends AppModel {

	var $name = 'Campaign';

  var $validate = array(
      'id' => VALID_NON_NEG_INT,
      'name' => VALID_NOT_EMPTY,
//      'orderer' =>
      'start_time' => VALID_DB_DATETIME,
      'end_time' => VALID_DB_DATETIME_OR_EMPTY,
//      'message_subject' =>
//      'message' =>
      'target_amount' => VALID_NON_NEG_INT,
      'accomplished_amount' => VALID_NON_NEG_INT,
      'user_client_id' => VALID_NON_NEG_INT,
      'created' => VALID_DB_DATETIME_OR_EMPTY,
      'modified' => VALID_DB_DATETIME_OR_EMPTY
  );

	var $hasAndBelongsToMany = array('User' =>
                               array('className'    => 'User',
                                    
                               )
	);
	
	var $hasMany = array('Classification' =>
                         array('className'     => 'Classification',
                               'conditions'    => '',
                               'order'         => '',
                               'limit'         => '',
                               'foreignKey'    => '',
                               'dependent'     => '',
                               'exclusive'     => '',
                               'finderQuery'   => ''
                         ),
						'SalesArgument' =>
                         array('className'     => 'SalesArgument',
                               'conditions'    => '',
                               'order'         => '',
                               'limit'         => '',
                               'foreignKey'    => '',
                               'dependent'     => '',
                               'exclusive'     => '',
                               'finderQuery'   => ''
                         ),
						'TargetClient' =>
                         array('className'     => 'TargetClient',
                               'conditions'    => '',
                               'order'         => '',
                               'limit'         => '',
                               'foreignKey'    => '',
                               'dependent'     => '',
                               'exclusive'     => '',
                               'finderQuery'   => ''
                         ),
						'MessageLink' =>
                         array('className'     => 'MessageLink',
                               'conditions'    => '',
                               'order'         => '',
                               'limit'         => '',
                               'foreignKey'    => '',
                               'dependent'     => '',
                               'exclusive'     => '',
                               'finderQuery'   => ''
                         ),
            'CampaignsUser' =>
                         array('className'     => 'CampaignsUser',
                               'conditions'    => '',
                               'order'         => '',
                               'limit'         => '',
                               'foreignKey'    => '',
                               'dependent'     => '',
                               'exclusive'     => '',
                               'finderQuery'   => ''
                         )
	);
	
    var $belongsTo = array('UserClient' =>
                           array('className'  => 'UserClient',
                                 'conditions' => '',
                                 'order'      => '',
                                 'foreignKey' => ''
                           )
    );
    
   /**
   * Palauttaa kampanjaan kuuluvan shkpostiviestin sislln myyjn tietojen kanssa.
   * @param int $campaign_id kampanjan tunnistetieto.
   * @param int $user_id kyttjn tunnistetieto.
   * @return array shkpostiviestin sislt.
   * <ul>
   *   <li>['campaigns']['message_subject'] string Shkpostiviestin otsikko.</li>
   *   <li>['campaigns']['message'] string Shkpostiviestin sislt.</li>
   *   <li>['users']['name'] string Myyjn nimi.</li>
   *   <li>['campaigns_users']['phone_number'] string Myyjn kampanjakohtainen puhelinnumero.</li>
   *   <li>['campaigns_users']['email_address'] string Myyjn kampanjakohtainen shkpostiosoite.</li>
   * </ul>
   */
    function showMessage($campaign_id, $user_id) {
    	
    	$result = $this->query("SELECT
								  users.name,
								  campaigns_users.phone_number,
								  campaigns_users.email_address,
  								  campaigns.message,
  								  campaigns.message_subject
					  			FROM
  								  users,
  								  campaigns_users,
  								  campaigns
					  			WHERE
  								  campaigns_users.campaign_id = $campaign_id
 					   			 AND
  								  campaigns_users.campaign_id = campaigns.id
 					   			 AND
  								  campaigns_users.user_id = $user_id
 					   			 AND
  								  campaigns_users.user_id = users.id;");
    	
    	return $result;
    	
    }
    
   /**
   * Palauttaa lhiosumat tiedoille jotka saadaan parametreina.
   * @param int $campaign_id kampanjan tunnistetieto.
   * @param string $y_id listtvn kohdeyrityksen y-tunnus.
   * @param string $name listtvn kohdeyrityksen nimi.
   * @return array kaikki lhiosumat.
   * <ul>
   *   <li>[0..n]['target_clients']['id'] int Tietueen ollessa lhiosuma, kentt sislt kohdeasiakkaan tunnistetiedon. Muussa tapauksessa (=uusi tietue) arvo on -1.</li>
   *   <li>[0..n]['target_clients']['y_id'] string Kohdeasiakkaan Y-tunnus.</li>
   *   <li>[0..n]['target_clients']['domain'] string Kohdeasiakkaan toimiala.</li>
   *   <li>[0..n]['target_clients']['exchange_number'] string Kohdeasiakkaan vaihteen puhelinnumero.</li>
   *   <li>[0..n]['target_clients']['address'] string Kohdeasiakkaan osoite.</li>
   *   <li>[0..n]['target_clients']['email_address'] string Kohdeasiakkaan shkpostiosoite.</li>
   *   <li>[0..n]['target_clients']['homepage'] string Kohdeasiakkaan kotisivu.</li>
   *   <li>[0..n]['target_clients']['inoa_id'] string Kohdeasiakkaan Inoa tunnistetieto.</li>
   *   <li>[0..n]['events']['contact_name'] string Yhteyshenkiln nimi.</li>
   *   <li>[0..n]['events']['contact_title'] string Yhteyshenkiln titteli.</li>
   *   <li>[0..n]['events']['contact_phone'] string Yhteyshenkiln puhelinnumero.</li>
   *   <li>[0..n]['events']['contact_email_address'] string Yhteyshenkiln shkpostiosoite.</li>
   * </ul>
   */
    function checkNearHits($campaign_id, $y_id, $name) {
    	
        $y_id_clause = '';
        if($y_id != ''){
          $y_id_clause = ' OR y_id = ' . $y_id;      
        }
        
    	$result = $this->query("SELECT
									  *
									FROM
									  target_clients AS TargetClient
									    INNER JOIN
										  campaigns_users ON TargetClient.campaigns_user_id = campaigns_users.id
									    INNER JOIN
										  users AS User ON User.id = campaigns_users.user_id
									    LEFT JOIN
										  latest_events AS LatestEvent ON LatestEvent.target_client_id = TargetClient.id
									WHERE
										TargetClient.campaign_id = $campaign_id
        							AND
										(TargetClient.name LIKE '%$name%'" . $y_id_clause . ")" );
									
    	return $result;
    	
    }

   /**
   * Tallentaa yksittisen kohdeasiakkaan tietokantaan ja luo samalla siit eventin
   * @param array $nearhits yksittinen listtv kohdeasiakas
   * @param int $campaign_id kampanjan tunnistetieto
   * @param int $replace_this korvattavan kohdeasiakkaan id
   * @param int $user_id kyttjn id
   * @param boolean $for_user listk uusi kohdeasiakas kyttjlle vai meneek pooliin
   *
   */
    
    function saveContact($nearhits, $campaign_id, $replace_this, $user_id, $for_user) {
    	
    	$name = $nearhits['TargetClient']['name'];
    	$y_id = $nearhits['TargetClient']['y_id'];
    	$homepage = $nearhits['TargetClient']['homepage'];
    	$inoa_id = $nearhits['TargetClient']['inoa_id'];
    	$exchange_number = $nearhits['TargetClient']['exchange_number'];
    	$address = $nearhits['TargetClient']['address'];
    	$domain = $nearhits['TargetClient']['domain'];
    	$email_address = $nearhits['TargetClient']['email_address'];
    	$contact_email_address = $nearhits['LatestEvent']['contact_email_address'];
    	$contact_name = $nearhits['LatestEvent']['contact_name'];
    	$contact_title = $nearhits['LatestEvent']['contact_title'];
    	$contact_phone_number = $nearhits['LatestEvent']['contact_phone_number'];
    	$user = $nearhits['User']['name'];
    	
    	$for_this_user = "NULL";
    	
    	if($for_user == true) {
    		$for_this_user = "'" . $this->getCampaignsUserId($user_id, $campaign_id) . "'";
    	    
    		// $for_this_user = "'" . $user_id . "'";
    	}
    	
    	if(!empty($replace_this)) {
    		$this->query("UPDATE target_clients SET
								name = '$name',y_id = '$y_id',domain = '$domain',exchange_number = '$exchange_number',
								address = '$address',email_address = '$email_address',
								homepage = '$homepage',inoa_id = '$inoa_id',created = '0000-00-00 00:00:00',
								modified = '0000-00-00 00:00:00'
							WHERE id = '$replace_this'");
    		
    		$result = $this->query("SELECT id FROM target_clients WHERE name = '$name' and campaign_id = '$campaign_id'");
			$insertthis = $result[0]['target_clients']['id'];

			$this->query("INSERT INTO events
								(id,start_time,end_time,contact_at,contact_name,contact_title,
								contact_phone_number,contact_email_address,created,modified,event_type_id,
								target_client_id,user_id
								)
							VALUES
								('',now(),now(),NULL,'$contact_name','$contact_title','$contact_phone_number',
								'$contact_email_address',now(),now(),'Kesken','$insertthis','$user_id')"
								);
    	}
    	else {
    		$this->query("INSERT INTO target_clients
								(
								id,name,y_id,domain,exchange_number,address,email_address,homepage,inoa_id,
								created,modified,campaign_id,target_client_state_id,campaigns_user_id
								)
							VALUES
								(
								'','$name','$y_id','$domain','$exchange_number','$address','$email_address',
								'$homepage','$inoa_id','0000-00-00 00:00:00','0000-00-00 00:00:00','$campaign_id',
								'Ei aloitettu',$for_this_user)"
								);
								
			$result = $this->query("SELECT id FROM target_clients WHERE name = '$name' and campaign_id = '$campaign_id'");
			$insertthis = $result[0]['target_clients']['id'];

			$this->query("INSERT INTO events
								(id,start_time,end_time,contact_at,contact_name,contact_title,
								contact_phone_number,contact_email_address,created,modified,event_type_id,
								target_client_id,user_id
								)
							VALUES
								('',now(),now(),NULL,'$contact_name','$contact_title','$contact_phone_number',
								'$contact_email_address',now(),now(),'Luotu','$insertthis','$user_id')"
								);
    	}
   }
    
    /**
     * Etsii kyttjn kohdeasiakkaita.
     * @param int $user_id kyttjn tunnistetieto.
     * @param postarray $searcharray hakuehdot:
     * <ul><li>[Campaign][id] int Kampanjan tunnistetieto. Arvo -1 hakee kaikista kampanjoista.</li>
     *     <li>[TargetClient][name] string Kohdeasiakkaan nimi.</li>
     *     <li>[TargetClient][exchange_number] string Yhteyshenkiln tai vaihteen puhelinnumero.</li>
     *     <li>[Event][contact_name] string Yhteyshenkiln nimi.</li>
     *     <li>[TargetClient][y_id] string Kohdeasiakkaan y-tunnus.</li>
     * </ul>
     * 
     * @return array hakutulokset:
     * <ul>
     *   <li>[0...n]['campaigns']['name'] string Kampanjan nimi.</li>
     *   <li>[0...n]['campaigns']['id'] int Kampanjan tunnistetieto.</li> 
     *   <li>[0...n]['target_clients']['name'] string Kohdeasiakkaan nimi.</li>
     *   <li>[0...n]['target_clients']['id'] int Kohdeasiakkaan tunnus.</li>
     *   <li>[0...n]['events']['contact_name'] string Yhteyshenkiln nimi.</li>
     *   <li>[0...n]['events']['contact_phone_number'] string Yhteyshenkiln puhelinnumero.</li>
     *   <li>[0...n]['target_clients']['exchange_number'] string Vaihteen puhelinnumero.</li>
     *   <li>[0...n]['users']['name'] string Myyjn nimi, jolle kohdeasiakas kuuluu.</li>
     * </ul>
     */
    function searchContacts($searcharray, $user_id) {
             
      $campaign_id = trim($searcharray['Campaign']['id']);
      $name = trim($searcharray['TargetClient']['name']);
      $exchange_number = trim($searcharray['TargetClient']['exchange_number']);
      $contact_name = trim($searcharray['Event']['contact_name']);
      $y_id = trim($searcharray['TargetClient']['y_id']);
      
      
      $campaign_id_clause = '';
      if ($campaign_id != -1) {
        $campaign_id_clause = " AND TargetClient.campaign_id = '$campaign_id' AND Campaign.id = '$campaign_id'";
      }
      
      else if($campaign_id == -1) {
        $campaign_id_clause = " AND Campaign.id = TargetClient.campaign_id";
      }
      
      $name_clause = '';
      if(!empty($name)) {
        $name_clause = " AND TargetClient.name LIKE '%$name%'";
      }
      
      $exchange_number_clause = '';
      if(!empty($exchange_number)) {
      $exchange_number = preg_replace('/[+\\-() ]/' ,'', $exchange_number);
      
        $en_array = str_split($exchange_number);
        //alussa saa olla kaikkia sallittuja merkkej
        $en_match ='\\\+?[0-9 ()-]*';
        //keskell () - ja vlilynti
        foreach($en_array as $a){
        $en_match = $en_match . $a . '[- ()]*';
        }
        //lopussa saa olla numeroita sek samoja kuin keskell
        $en_match = $en_match . '[0-9 ()-]*';
        $exchange_number_clause = " AND (TargetClient.exchange_number REGEXP '$en_match'
                                    OR Event.contact_phone_number REGEXP '$en_match')";
      }
      
      
      $contact_name_clause = '';
      if(!empty($contact_name)) {
        $contact_name_clause = " AND Event.contact_name LIKE '%$contact_name%'";
      }
      
      $y_id_clause = '';
      if(!empty($y_id)) {
        $y_id_clause = " AND TargetClient.y_id = '$y_id'";
      }
               
      $result = $this->query("SELECT DISTINCT  Campaign.name, Campaign.id, TargetClient.name, TargetClient.id, 
                                     TargetClient.exchange_number, Event.contact_name,
                                     Event.contact_phone_number, User.name 
                 FROM target_clients AS TargetClient, events AS Event, campaigns AS Campaign , users AS User 
                 WHERE TargetClient.id = Event.target_client_id
                   AND User.id = $user_id
                   AND TargetClient.campaigns_user_id = $user_id 
                   $campaign_id_clause  $name_clause  $exchange_number_clause  
                   $contact_name_clause  $y_id_clause
                   ORDER BY TargetClient.name
                ");
      
       return $result;
      
    }

}
?>