<?php
/**
 * CampaignsUser
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Models
 *
 *
 */
class CampaignsUser extends AppModel {

  var $name = 'CampaignsUser';

  var $validate = array(
      'id' => VALID_NON_NEG_INT,
      'user_id' => VALID_NON_NEG_INT,
      'campaign_id' => VALID_NON_NEG_INT,
      'phone_number' => VALID_PHONE_NUMBER_OR_EMPTY,
      'email_address' => VALID_EMAIL,
      'target_amount' => VALID_NON_NEG_INT,
      'accomplished_amount' => VALID_NON_NEG_INT,
      'calls_per_day' => VALID_NON_NEG_INT,
      'calls_per_day_suggestion' => VALID_NON_NEG_INT,
      'created' => VALID_DB_DATETIME_OR_EMPTY,
      'modified' => VALID_DB_DATETIME_OR_EMPTY
  );

  var $belongsTo = array('User' =>
  array('className'  => 'User',
  'conditions' => '',
  'order'      => '',
  'foreignKey' => ''
                           ),
  'Campaign' =>
  array('className'  => 'Campaign',
  'conditions' => '',
  'order'      => '',
  'foreignKey' => ''
                           )
  );

  /**
   * Palauttaa kohdeasiakkaiden lukumrn.
   *
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan. Arvo -1 palauttaa kaikkien kyttjien tiedot.
   * @param int $limit -1=kaikki, 0=keskeneriset, 1=valmiit
   *
   * @return int lukumr
   */
  function getTargetClientsCount($campaign_id, $user_id, $limit){

    /** Hakutaulukko*/

    $search_array = array();
    /** Mahdollinen kyttjrajaus */
    $user_limit = "";

    // Testaanpas kuuluuko kyttj thn kampanjaan
    if ($user_id<> -1){
      $campaigns_users_id = $this->getCampaignsUserId($user_id, $campaign_id);

      if ($campaigns_users_id == null){
        return 0;
      }
      $user_limit = "AND campaigns_user_id =" . $campaigns_users_id;
    }

    // Siis kuuluu
    switch ($limit) {
      case -1:
        $query = $this->query("select count(*) from target_clients where campaign_id = $campaign_id $user_limit");
        return $query[0][0]['count(*)'];
        ;
        break;

      case 0:
        $query = $this->query("select count(*) from target_clients where campaign_id = $campaign_id $user_limit AND target_client_state_id in ('Kesken', 'Ei aloitettu')");
        return $query[0][0]['count(*)'];
        break;

      case 1:
        $query = $this->query("select count(*) from target_clients where campaign_id = $campaign_id $user_limit AND target_client_state_id IN ('Tavoite saavutettu', 'Tavoitetta ei saavutettu')");
        return $query[0][0]['count(*)'];
    }


  }

  /**
   * Palauttaa tiedon tavoitesoittojen lukumrst.
   *
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan.
   *
   * @return int Tavoitesoittojen lukumr, null jos parametrit virheelliset tai kampanjatietoa ei ole
   */
  function getCallsPerDay($campaign_id, $user_id){
    $search_array['CampaignsUser.user_id'] = $user_id;
    $search_array['CampaignsUser.campaign_id'] = $campaign_id;
    $this->unbindModel(array('belongsTo' => array('User', 'Campaign')));
    $campaign_users = $this->find($search_array);
    return $campaign_users['CampaignsUser']['calls_per_day'];
  }

  /**
   * Palauttaa kutsupivn tehtyjen soittojen lukumrn.
   *
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan.
   *
   * @return int Tnn tehtyjen soittojen lukumr, null, jos liitosta ei ole.
   */

  function getCallsDoneToday($campaign_id,$user_id){
    // Haetaan campaigns__user-id
    $campaigns_user_id = $this->getCampaignsUserId($user_id, $campaign_id);

    // Jos liitosta ei ole, niin palautetaan null
    if ($campaigns_user_id == null){

      return null;
    }
    $query = $this->query("SELECT count(Event.id) from events as Event
				 INNER JOIN target_clients as TargetClient ON TargetClient.id = Event.target_client_id
				 WHERE TargetClient.campaigns_user_id = $campaigns_user_id 
					and date(Event.start_time) = curdate()
					and Event.event_type_id in (
					SELECT event_types.id from event_types
														INNER JOIN event_type_classes_event_types 
														ON event_types.id = event_type_classes_event_types.event_type_id 
														INNER JOIN event_type_classes 
														ON event_type_classes.id = event_type_classes_event_types.event_type_class_id where event_type_classes.id in ('Soitto'))", false);

    return $query[0][0]['count(Event.id)'];
  }

  /**
   * Palauttaa kyttjn ja kyttjasiakkaan kampanjat.
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan.
   * @param int $user_client_id Kyttjasiakkaan tunnistetieto. Jos arvo on -1, haetaan vain kyttjn kampanjat.
   *
   * @return array Kampanjat, null jos kyttjll ei ole kampanjoita tai kyttjn id-numeroa ei ole
   * <ul>
   *  <li>[0..n][Campaign][id]</li>
   *  <li>[0..n][Campaign][name]</li>
   *  <li>[0..n][Campaign][start_time]</li>
   *  <li>[0..n][Campaign][end_time]</li>
   *  <li>[0..n][CampaignsUser][id]</li>
   *  <li>[0..n][CampaignsUser][user_id]</li>
   *  <li>[0..n][TargetClient][contact_at_count] tulevien soittojen lukumr</li>
   *  <li> [0..n][Event][contact_at] Seuraavan soiton aika, jos olemassa.</li>
   *
    
   * </ul>
   */
  function getUserCampaigns($user_id, $user_client_id = -1){
    /** palautusarvot */
    $return_values = null;

    //Haetaan kaikki kyttjn kampanjat
    //TODO mietittv jrjestys tarkemmin
    $query = $this->query("
        SELECT DISTINCT Campaign.id, Campaign.name, Campaign.start_time, Campaign.end_time, CampaignsUser.id, CampaignsUser.user_id
          FROM campaigns as Campaign
				  LEFT JOIN campaigns_users AS CampaignsUser
            ON Campaign.id = CampaignsUser.campaign_id
              AND CampaignsUser.user_id = $user_id
					WHERE CampaignsUser.user_id = $user_id
            OR Campaign.user_client_id = $user_client_id
          ORDER BY
            CampaignsUser.user_id = $user_id DESC,
            Campaign.start_time > curdate() DESC,
            Campaign.end_time IS NULL OR Campaign.end_time > curdate() DESC,
            Campaign.end_time ASC,
            Campaign.start_time DESC,
            Campaign.name ASC
      ");

    // loopataan nihin kiinni tulevien soittojen tiedot
    foreach ($query as $queryrow) {
      // Haetaan kampanjaliitoksen id
      $campaigns_user_id = $queryrow['CampaignsUser']['id'];
      //      $campaigns_user_id = $this->getCampaignsUserId($user_id, $queryrow['Campaign']['id']);
      if (! empty($campaigns_user_id)) {
        // Haetaan luku
        $contact_at_query = $this->query("SELECT count(contact_at) from latest_events INNER JOIN target_clients ON latest_events.target_client_id = target_clients.id
										WHERE target_clients.campaigns_user_id =  $campaigns_user_id AND contact_at <> '0000-00-00 00:00:00' ");

        $queryrow['TargetClient']['contact_at_count'] = $contact_at_query[0][0]['count(contact_at)'];
         
        // Haetaan seuraava, joka on siis pienin kaikista latest_eventeist, joissa contact_at <> 0
        $next_contact_query = $this->query("select contact_at FROM latest_events
										INNER JOIN target_clients ON latest_events.target_client_id = target_clients.id 
											WHERE target_clients.campaigns_user_id =  $campaigns_user_id
											AND contact_at <> '0000-00-00 00:00:00' 
											AND contact_at in (SELECT min(contact_at) FROM latest_events 
																INNER JOIN target_clients ON latest_events.target_client_id = target_clients.id 
																WHERE target_clients.campaigns_user_id =  $campaigns_user_id AND contact_at <> '0000-00-00 00:00:00')
										");

        // Liitetn, jos tuli jotain
        if(!empty($next_contact_query)){
          $queryrow['Event']['contact_at'] = $next_contact_query[0]['latest_events']['contact_at'];
        } else {
          $queryrow['Event']['contact_at'] = null;
        }
      }

      $return_values[] = $queryrow;

    }

    //    if (DEBUG > 0) print_r($return_values); die();

    return $return_values;
  }

  /**
   * Palauttaa kaikki kampanjaan kuuluvat kohdeasiakkaat.
   * @param int $user_id Kyttjn tunnistetieto. Arvo -1 palauttaa kaikkien kyttjien tiedot.
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param int $limit -1=palauttaa kaikki; 0=keskeneriset; 1=vain ksitellyt.
   *
   * @return array Kampanjan kohdeasiakkaat, null jos kohdeasiakkaita ei ole.
   * <ul>
   *   <li>[0..n][TargetClient] array Kohdeasiakkaan tiedot.</li>
   *   <li>[0..n][Event] array Viimeisimmn tapahtuman tiedot.</li>
   *   <li>[0..n][allow_email_sending] boolean Saako kohdeasiakkaalle lhett shkpostia.
   *       Lhettminen estetn, mikli kohdeasiakkaan nykyiseen shkpostiosoitteeseen on
   *       jo lhetetty viesti.</li>
   * </ul>
   */
  function getCampaignList($user_id, $campaign_id, $limit){
    /** Kyttjrajoituksen sql-osuus*/
    $user_limit  = "";
    /** Kohdeasiakkaan tilarajoitus */
    $target_client_limit = "";
    /** Palautustaulukko */
    $return_array = null;

    if ($user_id <> -1){
      $campaign_users_id = $this->getCampaignsUserId($user_id, $campaign_id);
      // Jos ei liitosta -> null
      if ($campaign_users_id == null){
        echo "null";
        return null;
      }
      $user_limit = "AND TargetClient.campaigns_user_id = $campaign_users_id";
       
    }

    //Tehdn tilarajoitus
    switch ($limit) {
      case 0:
        $target_client_limit = "AND TargetClient.target_client_state_id in ('Kesken', 'Ei aloitettu')";
        break;
      case 1:
        $target_client_limit = "AND TargetClient.target_client_state_id in ('Tavoite saavutettu', 'Tavoitetta ei saavutettu')";
        break;

    }
     
    //TODO Ordering
    $target_clients_query = $this->query("SELECT TargetClient.id, TargetClient.name, Event.contact_name, Event.contact_at, Event.start_time,
					Event.event_type_id, Event.interest, Event.contact_email_address, events.event_type_id, events.start_time, events.contact_email_address 
					FROM target_clients as TargetClient  
					LEFT JOIN latest_events AS Event ON Event.target_client_id = TargetClient.id
					LEFT JOIN (select * from events
					WHERE events.event_type_id in (SELECT event_types.id from event_types
														INNER JOIN event_type_classes_event_types 
														ON event_types.id = event_type_classes_event_types.event_type_id 
														WHERE event_type_classes_event_types.event_type_class_id in ('Viesti'))) as events
					ON events.contact_name = Event.contact_name WHERE 1 = 1 $user_limit $target_client_limit
					and TargetClient.campaign_id = $campaign_id"); 

    // Jos ei asiakkaita -> null
    if ($target_clients_query == null){
      return null;
    }
    // Loopataan ja tehdn ntiksi
    $return_array;
    $target_client_id = -1;
    $i = 0;

    foreach ($target_clients_query as $target_clients_row) {
      // Tll hoidetaan shkpostilinkit ntisti osaksi structia
      if ($target_client_id == $target_clients_row['TargetClient']['id']){
        $return_array[$i-1]['Event']['email_events'][] = $target_clients_row['events'];
        continue;
      }

      $return_array[$i] = $target_clients_row;
      $return_array[$i]['Event']['email_events'][] = $target_clients_row['events'];

      if ($target_clients_row['Event']['contact_email_address'] == "" || $target_clients_row['Event']['contact_email_address'] <> null && $target_clients_row['Event']['contact_email_address'] == $target_clients_row['events']['contact_email_address']){
        $return_array[$i]['allow_email_sending'] = false;
      } else {
        $return_array[$i]['allow_email_sending'] = true;
      }
       
      $target_client_id = $target_clients_row['TargetClient']['id'];
      ++$i;
    }
     
    return $return_array ;
  }

  /**
   * Palauttaa kaikki kampanjaan kuuluvat kohdeasiakkaat.
   * @param int $campaign_id Kampanjan tunnistetieto.
   *
   * @return array Kampanjan kohdeasiakkaat, null jos kohdeasiakkaita ei ole
   * <ul>
   *   <li>[0..][TargetClient][id] int Kohdeasiakkaan tunnistetieto.</li>
   *   <li>[0..][TargetClient][name] string Kohdeasiakkaan nimi.</li>
   *   <li>[0..][TargetClient][target_client_state] string Kohdeasiakkaan tila.</li>
   *   <li>[0..][User][id] int Kyttjn tunnistetieto, johon kohdeasiakas liitetn.</li>
   * <ul>
   */
  function getUsersTargets($campaign_id){

    $query =  $this->query("SELECT TargetClient.id, TargetClient.name, TargetClient.target_client_state_id, User.id
				  FROM target_clients as TargetClient LEFT JOIN campaigns_users as CampaignsUser ON TargetClient.campaigns_user_id = CampaignsUser.id 
					LEFT JOIN users as User ON User.id = CampaignsUser.user_id where TargetClient.campaign_id = $campaign_id");
    if ($query == null){
      return null;
    } else {
      return $query;
    }


  }



}
?>
