<?php
/**
 * ClassificationsTargetClients
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Models
 */
class ClassificationsTargetClients extends AppModel {

  var $name = 'ClassificationsTargetClients';

  var $validate = array(
      'target_client_id' => VALID_NON_NEG_INT,
      'classification_id' => VALID_NON_NEG_INT
  );

  var $belongsTo = array('Classification' =>
                           array('className'  => 'Classification',
                                 'conditions' => '',
                                 'order'      => '',
                                 'foreignKey' => ''
                           )
  );
  
  var $hasAndBelongsToMany = array('TargetClient' =>
                               array('className'    => 'TargetClient',
                                     'joinTable'    => '',
                                     'foreignKey'   => '',
                                     'associationForeignKey'=> '',
                                     'conditions'   => '',
                                     'order'        => '',
                                     'limit'        => '',
                                     'unique'       => '',
                                     'finderQuery'  => '',
                                     'deleteQuery'  => '',
                               )
  );

    
  /**
   * Palauttaa raportin kampanjaan kuuluvien kohdeasiakkaiden luokittelutiedoista.
   *
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan. Arvo -1 palauttaa
   * kaikkien kyttjien tiedot.
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param DateString $start_time = null. Rajauksen aloitusaika.
   * @param DateString $end_time = null. Rajauksen lopetusaika.
   * 
   * @return array Raportti taulukko.
   * <ul>
   *   <li>[0..n][Classification][classification] string Luokittelutiedon nimi.</li>
   *   <li>[0..n][TargetClientClassification][count] int Luokittelutiedon lukumr.</li>
   *   <li>[0..n][TargetClientClassification][percentage_all] int Prosenttia kaikista.</li>
   *   <li>[0..n][TargetClientClassification][percentage_called] int Prosenttia soitetuista.</li>
   * </ul>
   */
    function getTargetClientsClassificationReport($user_id, $campaign_id, $start_time = null, $end_time = null) {
    // Hakukyselyn aikarajausosio
    $aikarajaus = "";
    /** Kampanja id, joka pit hakea kyselyn kautta */
    $campaigns_user_id = 0;
    /** Kampanjaan kuuluvien kaikkien asiakkaiden lukumr */
    $kaikkiasiakkaat = 0;
    /** Asiakkaat, jotka kuuluvat rajaukseen */
    $asiakasrajauslkm = 0;
    //Kyttjn rajoitus kyselyiss
    $kayttajarajoitus = "";
    //Palaute
    $palauterivi = array();

    //Mahdollinen kyttjrajoitus
    if ($user_id <> -1){
      $campaigns_user_id = $this->getCampaignsUserId($user_id,$campaign_id);
      $kayttajarajoitus = "AND target_clients.campaigns_user_id = $campaigns_user_id";
    }

    //Mahdollinen aikarajaus
    if ($start_time !== null && $end_time === null ){
      $aikarajaus = "AND events.start_time >= '$start_time'";
    } else if ($start_time !== null && $end_time !== null){
      $aikarajaus = "AND events.start_time BETWEEN '$start_time' AND '$end_time'";
    }

    //Haetaan kaikkien asiakkaiden lukumr
    $kysely = $this->query("SELECT COUNT(*)
              FROM target_clients 
              WHERE campaign_id = $campaign_id
                $kayttajarajoitus
              " );
    $kaikkiasiakkaat = $kysely[0][0]['COUNT(*)'];
    if($kaikkiasiakkaat === '0') {$kaikkiasiakkaat = 1;}

    //Haetaan ne, jotka kuuluvat rajaukseen    
    if ($user_id <> -1){
      $hakuarray['TargetClient.campaigns_user_id'] = $campaigns_user_id;
    }
    $hakuarray['TargetClient.campaign_id'] = $campaign_id;
    
    //Haetaan kohteet
    $kysely_vanh = $this->query("SELECT COUNT(*)
							FROM target_clients
               				WHERE target_clients.campaign_id = $campaign_id
                            AND target_clients.id IN 
          							(SELECT target_client_id FROM events as events2
          							WHERE (start_time) IN 
            							(SELECT max(start_time) 
            							FROM events, target_clients
                                        WHERE target_clients.campaign_id = $campaign_id
            							AND events.target_client_id = target_clients.id
										and events.target_client_id = events2.target_client_id 
                                   		$aikarajaus 
               							$kayttajarajoitus
                                        AND events.event_type_id IN
                							(SELECT event_types.id 
											FROM event_types, event_type_classes_event_types, event_type_classes 
              								WHERE event_type_classes_event_types.event_type_id = event_types.id
                                            AND event_type_classes.id = event_type_classes_event_types.event_type_class_id
											AND event_type_classes.id = 'Soitto') 
                 					)
                            )");
    
    $event_type_class_query = "SELECT event_types.id from event_types 
                                INNER JOIN event_type_classes_event_types 
                                        ON event_types.id = event_type_classes_event_types.event_type_id 
                                INNER JOIN event_type_classes 
                                        ON event_type_classes.id = event_type_classes_event_types.event_type_class_id";
    // $limit_in = "and Event.event_type_id in ($event_type_class_query where event_type_classes.id in ($luokkarajaus))";
    
    $kysely = $this->query("SELECT count(*) from events as Event
					INNER JOIN target_clients ON Event.target_client_id = target_clients.id 
					INNER JOIN 
								(select target_client_id, max(start_time) as time 
									FROM events  
									WHERE events.event_type_id IN
                							(SELECT event_types.id 
											FROM event_types, event_type_classes_event_types, event_type_classes 
              								WHERE event_type_classes_event_types.event_type_id = event_types.id
                                            AND event_type_classes.id = event_type_classes_event_types.event_type_class_id
											AND event_type_classes.id = 'Soitto') $aikarajaus 
									group by target_client_id) as latest_event ON latest_event.time = Event.start_time and latest_event.target_client_id = target_clients.id   
						AND campaign_id = $campaign_id
    					$kayttajarajoitus");
    
    $asiakasrajauslkm = $kysely[0][0]['count(*)'];  
    if($asiakasrajauslkm === '0') {$asiakasrajauslkm = 1;}     
    
    //Haetaan kohteet
    $tulos = $this->query("SELECT classification, COUNT(*) AS lkm 
							FROM target_clients, classifications_target_clients, classifications 
               			  	WHERE target_clients.campaign_id = $campaign_id
                            AND target_clients.id IN 
                 				(SELECT target_client_id FROM events 
                  				WHERE (target_client_id, start_time) IN 
                      					(SELECT target_client_id, max(start_time) 
                       					FROM events 
                                        WHERE target_clients.campaign_id = $campaign_id
                                         $aikarajaus 
                         				 $kayttajarajoitus
                                         GROUP BY target_client_id))
                				AND classifications_target_clients.target_client_id = target_clients.id  
                 				AND classifications.id = classifications_target_clients.classification_id 
               				GROUP BY classifications.classification");
    
    // Luodaan palautusarray
      foreach ($tulos as $rivi ) {
        $palaute['Classification']['classification'] = $rivi['classifications']['classification'];
        $palaute['TargetClientClassification']['count'] = $rivi[0]['lkm'];
        $palaute['TargetClientClassification']['percentage_all'] = ($rivi[0]['lkm'] / $kaikkiasiakkaat)*100;
        $palaute['TargetClientClassification']['percentage_called'] = ($rivi[0]['lkm'] / $asiakasrajauslkm)*100;
        $palauterivi[] = $palaute;
  }
  return $palauterivi;
  }
}
?>
