<?php
/**
 * Event
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Models
 */
class Event extends AppModel {

  var $name = 'Event';
  
  var $validate = array(
    'id' => VALID_NON_NEG_INT,
    'start_time' => VALID_DB_DATETIME,
    'end_time' => VALID_DB_DATETIME,
    'contact_at' => VALID_DB_DATETIME_OR_EMPTY,
    'interest' => VALID_INTEGER,
//    'comment' => VALID_,
    'contact_name' => VALID_NOT_EMPTY,
//    'contact_title' =>  VALID_,
    'contact_phone_number' => VALID_PHONE_NUMBER_OR_EMPTY,
    'contact_email_address' =>VALID_EMAIL_OR_EMPTY,
    'created' => VALID_DB_DATETIME_OR_EMPTY,
    'modified' => VALID_DB_DATETIME_OR_EMPTY,
    'event_type_id' => VALID_NOT_EMPTY,
    'target_client_id' => VALID_NON_NEG_INT,
    'user_id' => VALID_NON_NEG_INT,
    'sales_argument_id' => VALID_NON_NEG_INT_OR_EMPTY
//    'reason_no_interest_id' => VALID_
  );

  var $belongsTo = array('User' =>
  array('className'  => 'User'
                           ),
  'SalesArgument' =>
  array('className'  => 'SalesArgument'
                           ),
  'TargetClient' =>
  array('className'  => 'TargetClient'
                           ),
  'EventType' =>
  array('className'  => 'EventType'
                           ),
  'ReasonNoInterest' =>
  array('className'  => 'ReasonNoInterest'
                           )
  );

  var $hasAndBelongsToMany = array('MessageLink' =>
  array('className'    => 'MessageLink',
  'joinTable'    => '',
  'foreignKey'   => 'user_event_id',
  'associationForeignKey'=> '',
  'conditions'   => '',
  'order'        => '',
  'limit'        => '',
  'unique'       => '',
  'finderQuery'  => '',
  'deleteQuery'  => '',
  )
  );

  /**
   * Palauttaa kohdeasiakkaaseen liittyvn viimeisimmn tapahtuman tiedot.
   * Virheellisen syteparametrin tai kannasta puuttuvan tunnistetiedon tapauksessa palauttaa false.
   *
   * @param int $target_client_id
   * 
   * @return array Viimeisimmn tapahtuman tiedot
   * <ul>
   *   <li>[Event][id] int Tapahtuman tunnistetieto.</li>
   *   <li>[Event][start_time] DateString Tapahtuman aloitusaika.</li>
   *   <li>[Event][end_time] DateString Tapahtuman lopetusaika.</li>
   *   <li>[Event][contact_at] DateString Seuraavan yhteydenoton ajankohta. Arvo on null, jos ei ole asetettu.</li>
   *   <li>[Event][interest] int Kiinnostus.</li>
   *   <li>[Event][comment] string Kommentti.</li>
   *   <li>[Event][contact_name] string Yhteyshenkiln nimi.</li>
   *   <li>[Event][contact_title] string Yhteyshenkiln titteli.</li>
   *   <li>[Event][contact_phone_number] string Yhteyshenkiln puhelinnumero.</li>
   *   <li>[Event][contact_email_address] string Yhteyshenkiln shkpostiosoite.</li>
   *   <li>[Event][created] DateString Luontipiv.</li>
   *   <li>[Event][modified] DateString Viimeinen muutospiv.</li>
   *   <li>[Event][event_type_id] string Tapahtumatyypin nimi.</li>
   *   <li>[Event][target_client_id] int Kohdeasiakkaan tunnistetieto.</li>
   *   <li>[Event][user_id] int Kyttjn tunnistetieto.</li>
   *   <li>[Event][sales_argument_id] int Myyntiargumentien tunnistetieto.</li>
   *   <li>[Event][reason_not_interest_id] string Kiinnostumattomuuden syy.</li>
   * </ul>  
   * 
   */
  function getLatestEvent($target_client_id){
    $hakuarray = array("Event.target_client_id" => $target_client_id);
    $hakuarray[] = "Event.start_time in (select max(start_time) from events where target_client_id = " . $target_client_id . ")";
    return $this->find($hakuarray);
  }

  /**
   * Palauttaa kohdeasiakkaan tapahtumahistorian. Virheellisten parametrien tapauksessa tai kannasta puuttuvan
   * tunnistetiedon tapauksessa palauttaa false.
   *
   * @param int $target_client_id Kohdeasiakkaan tunnistetieto.
   * @param DateString $start_time = null Rajauksen aloituspiv. 
   * @param DateString $end_time = null Rajauksen lopetuspiv.
   * 
   * @return array, Tapahtumahistoria-taulukko. Voi olla tyhj.
   * <ul>
   *   <li>[0..n][Event][id] int Tapahtuman tunnistetieto.</li>
   *   <li>[0..n][Event][start_time] DateString Tapahtuman aloitusaika.</li>
   *   <li>[0..n][Event][end_time] DateString Tapahtuman loptetusaika.</li>
   *   <li>[0..n][Event][contact_at] DateString Seuraavan yhteydenoton ajankohta. Arvo on null, jos ei ole asetettu.</li>
   *   <li>[0..n][Event][interest] int Kiinnostus.</li>
   *   <li>[0..n][Event][comment] string Kommentti.</li>
   *   <li>[0..n][Event][contact_name] string Yhteyshenkiln nimi.</li>
   *   <li>[0..n][Event][contact_title] string Yhteyshenkiln titteli.</li>
   *   <li>[0..n][Event][contact_phone_number] string Yhteyshenkiln puhelinnumero.</li>
   *   <li>[0..n][Event][contact_email_address] string Yhteyshenkiln shkpostiosoite.</li>
   *   <li>[0..n][Event][created] string Luontipiv.</li>
   *   <li>[0..n][Event][modified] string Viimeinen muutospiv.</li>
   *   <li>[0..n][Event][event_type_id] string Tapahtumatyypin nimi.</li>
   *   <li>[0..n][Event][target_client_id] string Kohdeasiakkaan tunnistetieto.</li>
   *   <li>[0..n][Event][user_id] string Kyttjn tunnistetieto.</li>
   *   <li>[0..n][Event][sales_argument_id] string Myyntiargumentin tunnistetieto.</li>
   *   <li>[0..n][Event][reason_not_interest_id] string Kiinnostumattomuuden syy.</li>
   *   <li>[0..n][SalesArgument][argument] string Myyntiargumentti.</li>
   *   <li>[0..n][MessageLink][0..n][target_url] string Linkin kohdeosoite.</li>
   *   <li>[0..n][MessageLink][0..n][target_url_text] string Linkin teksti.</li>
   * </ul>
   */
  function getTargetClientsEvents($target_client_id, $start_time = null, $end_time = null){
    $this->unbindModel(array('belongsTo' => array('User', 'ReasonNoInterest', 'TargetClient', 'EventType')));
    //Jos aloitusaika
    if ($start_time !== null && $end_time === null ){
      $hakuarray['date(Event.start_time)'] = ">=$start_time";
    } else if ($start_time !== null && $end_time !== null){
      $hakuarray['date(Event.start_time)'] = "between $start_time AND $end_time";
    }

    //Listn id
    $hakuarray['Event.target_client_id'] = $target_client_id;
    //Haetaan ja palautetaan

    return $this->findAll($hakuarray, null, null,0);
  }

  /**
   * Palauttaa arvon, kuinka monta kohdeasiakasta on klikannut jotain viestiss 
   * olleista linkeist. Eli yhteislukumr kyttjist, joilla on olemassa
   * vhintn yksi linkin klikkaustapahtuma. Virheellisten parametrien tapauksessa tai kannasta puuttuvan
   * tunnistetiedon tapauksessa palauttaa false.
   *
   *
   * @param int $user_id Kyttjn, joka kuuluu annettuun kampanjaan, tunnistetieto.
   * @param int $campaign_id Kampanjan, josta tieto halutaan, ja jonka myyj kyttj on, tunnistetieto.
   * 
   * @return int Viestin linkki klikanneiden kohdeasiakkaiden lukumr. 0 jos parametrit ei tsm.
   */

  function getDifferentEmailCounts($user_id, $campaign_id){
    //Tarkistetaanpas ekana onko tm kyttj mahdollisesti tss kampanjassa
	$query = $this->query("select count(distinct target_client_id) as lkm from events INNER JOIN target_clients ON target_clients.id = events.target_client_id where event_type_id = 'Klikattu' and target_clients.campaign_id = $campaign_id and target_clients.campaigns_user_id = $user_id;");	
	return $query[0][0]['lkm'];
  }

  /**
   *     Palauttaa niiden kohdeasiakkaiden lukumrn, jotka tyttvt/eivt
   tyt parametrina annetun tapahtumatyypin mukaisia ehtoja. POTENTIAALISESTI TURHA
   *
   *
   * @param int $user_id Kyttjn, jonka kampanjaan kuuluvien kohdeasiakkaiden lukumr halutaan, 
   * tunnistetieto. Kuuluttava kampanjaan. Arvo -1 palauttaa kaikkien kyttjien tiedot.
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param array $events [0..n] Tapahtuma-taulukko.
   * <ul>
   *   <li>[0..n][name] string Tapahtuman nimi taulussa event_types.
   * </ul>
   * @param DateString $start_time = null Rajauksen aloitusaika.
   * @param DateString $end_time = null Rajauksen lopetusaika.
   * @param boolean $include = true true=haetaan ne, jotka tyttvt hakuehdot. false=haetaan ne, 
   * jotka eivt tyt hakuehtoja.
   * 
   * @return int Kohdeasiakkaiden lukumr, jotka tyttvt/eivt tyt parametrina annetun 
   * tapahtumatyypin mukaisia ehtoja.
   */
  function getTargetClientCountByEventType($user_id, $campaign_id, $events, $start_time = null, $end_time = null, $include = true){
    //TODO enemmn kamaa testausta varten
    //Haetaanpas campaigns_user_id
    $campaigns_id = $this->getCampaignsUserId($user_id, $campaign_id);
    //Sitten luodaan kysely
    $hakuarray['TargetClient.campaigns_user_id'] = $campaigns_id;
    $hakuarray['TargetClient.campaign_id'] = $campaign_id;

    if ($include){
      $hakuarray['Event.event_type_id'] = $events;  
    } else {
      $first = true;
      $punctquery = "";
      foreach ($events as $event) {
      	if ($first){
          $punctquery = $punctquery . '$event';
      	  $first = false;
      	} else {
      	  $punctquery = $punctquery . ',$event';
      }
      
      }
      $hakuarray[] = "Event.event_type_id not like $events";
    }
    
    
    // Tss hakuhaarukka
    if ($start_time != null && $end_time == null ){
      $hakuarray['date(Event.start_time)'] = ">=$start_time";
    } else if ($start_time != null && $end_time != null){
      $hakuarray['date(Event.start_time)'] = "between $start_time AND $end_time";
    }
    
    $kysely = $this->find($hakuarray, "count(DISTINCT target_client_id)");
    return $kysely[0]['count(DISTINCT target_client_id)'];

  }

  /**
   * Palauttaa raportin kampanjaan kuuluvista kohdeasiakkaisiin liittyvist tapahtumista. 
   * Raportti lasketaan viimeisen raportoitava-muuttujan mrittmn tapahtumatyypin mukaisista arvoista.
   * Virheellisen syteparametrin tai kannasta puuttuvan tunnistetiedon tapauksessa palauttaa false.
   *
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan. Arvo -1 palauttaa kaikkien 
   * kyttjien tiedot
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param array $event_type_classes Halutut tapahtumaluokat -taulukko.
   * <ul>
   *   <li>[0..n] string Tapahtumaluokan nimi.</li>
   * </ul>
   * @param DateString $start_time = null Rajauksen aloitusaika.
   * @param DateString $end_time = null Rajauksen lopetusaika.
   * 
   * @return array
   * <ul>
   *   <li>[0..n][event_type_id] string Shkpostin tapahtumatyypin nimi.</li>
   *   <li>[0..n][count] int Tilan lukumr.</li>
   *   <li>[0..n][percentage_all] int Prosenttia kaikista.</li>
   *   <li>[0..n][percentage_called] int Prosenttia lhetetyist shkposteista.</li>
   * </ul>
   */
  function getClientEventTypesReport($user_id, $campaign_id, $event_type_classes, $start_time = null, $end_time = null){
    
    
    // Hakukyselyn aikarajausosio
    $aikarajaus = "";
    $aikarajaus2 = "";
    /** Kampanja id, joka pit hakea kyselyn kautta */
    $campaigns_user_id = 0;
    /** Kampanjaan kuuluvien kaikkien asiakkaiden lukumr */
    $kaikkiasiakkaat = 0;
    /** Asiakkaat, jotka kuuluvat rajaukseen */
    $asiakasrajauslkm = 0;
    //Luokkien rajaus kyselyiss
    $luokkarajaus = "";
    //Kyttjn rajoitus kyselyiss
    $kayttajarajoitus = "";

    //Haetaanpas campaigns_user_id


    //Mahdollinen kyttjrajoitus
    if ($user_id <> -1){
      $campaigns_user_id = $this->getCampaignsUserId($user_id,$campaign_id);
      $kayttajarajoitus = "AND target_clients.campaigns_user_id = $campaigns_user_id";
    }
    //Luodaan luokkarajaus
    $ensimminen = 0;
    foreach ($event_type_classes as $event_type_class) {
      if ($ensimminen <> 0){
        $luokkarajaus = $luokkarajaus . ",";
      }
      $luokkarajaus = $luokkarajaus . "\"$event_type_class\"";

    }
  
  // Aikarajaus
    if ($start_time !== null && $end_time === null ){
      $aikarajaus = "AND date(events.start_time) >= '$start_time'";
      $aikarajaus2 = "AND date(events2.start_time) >= '$start_time'";
    } else if ($start_time !== null && $end_time !== null){
      $aikarajaus = "AND date(events.start_time) BETWEEN '$start_time' AND '$end_time'";
      $aikarajaus2 = "AND date(events2.start_time) BETWEEN '$start_time' AND '$end_time'";
    }
    
    //Haetaan kaikkien asiakkaiden lukumr
    $kysely = $this->query("SELECT COUNT(*)
							FROM target_clients 
							WHERE campaign_id = $campaign_id
    						$kayttajarajoitus
							" );
    $kaikkiasiakkaat = $kysely[0][0]['COUNT(*)'];
    
    //Haetaan ne, jotka kuuluvat rajaukseen    
    if ($user_id <> -1){
      $hakuarray['TargetClient.campaigns_user_id'] = $campaigns_user_id;
    }
    $event_type_class_query = "SELECT event_types.id from event_types 
                                INNER JOIN event_type_classes_event_types 
                                        ON event_types.id = event_type_classes_event_types.event_type_id 
                                INNER JOIN event_type_classes 
                                        ON event_type_classes.id = event_type_classes_event_types.event_type_class_id";
    $limit_in = "and Event.event_type_id in ($event_type_class_query where event_type_classes.id in ($luokkarajaus))";
    
    $limit_query_vanh = "SELECT count(*) from events as Event
					INNER JOIN target_clients ON Event.target_client_id = target_clients.id 
					WHERE (start_time) IN 
									(select max(start_time) 
									FROM events 
									where events.target_client_id = target_clients.id and event_type_id in($event_type_class_query  where event_type_classes.id in ($luokkarajaus)  )
									$aikarajaus 
									) $limit_in 
						AND campaign_id = $campaign_id
    					$kayttajarajoitus";    
    
    $limit_query = "SELECT count(*) from events as Event
					INNER JOIN target_clients ON Event.target_client_id = target_clients.id 
					INNER JOIN 
								(select target_client_id, max(start_time) as time 
									FROM events 
									 where event_type_id in($event_type_class_query  where event_type_classes.id in ($luokkarajaus)  )
									$aikarajaus 
									group by target_client_id) as latest_event ON latest_event.time = Event.start_time and latest_event.target_client_id = target_clients.id   
						AND campaign_id = $campaign_id
    					$kayttajarajoitus";    
    
    
    $kysely = $this->query($limit_query);
    $asiakasrajauslkm = $kysely[0][0]['count(*)'];
    
    //haetaan kohteet 
    	$tulos = $this->query("SELECT event_types.id, alakysely.lkm AS lkm
                                FROM event_types
                                LEFT JOIN 
                                     (SELECT event_type_id, COUNT(event_type_id) AS lkm
                                        FROM events 
                                       INNER JOIN target_clients on events.target_client_id = target_clients.id   
                                             WHERE events.event_type_id IN ($event_type_class_query WHERE event_type_classes.id IN ($luokkarajaus)) 
                                               AND (start_time) IN 
                                                   (SELECT  MAX(start_time) 
                                                      FROM events as events2 
                                                     WHERE events2.target_client_id = events.target_client_id and event_type_id IN ($event_type_class_query WHERE event_type_classes.id IN ($luokkarajaus)  )
                                                           $aikarajaus2 
                                                           $kayttajarajoitus
                                                     GROUP BY target_client_id)
                                               AND target_clients.campaign_id = $campaign_id
                                             GROUP BY event_type_id) AS alakysely on alakysely.event_type_id = event_types.id 
                                      INNER JOIN event_type_classes_event_types ON event_type_classes_event_types.event_type_id = event_types.id  WHERE event_types.id IN ($event_type_class_query WHERE event_type_classes.id IN ($luokkarajaus))
                               GROUP BY event_types.id ORDER BY event_type_classes_event_types.order 
                                "); 

    // Otetaan selville, montako ei ole viel aloitettu
    $count = 0;
    $count_not_started = 0;
    foreach ($tulos as $rivi ) { 
     	if ($rivi['alakysely']['lkm'] == null){
     	  $count = 0;
     	} else {
     	  $count = $rivi['alakysely']['lkm']; 
     	}
        $count_not_started +=$count;
    }

    // Listn ensimminen rivi
        $palaute['Event']['event_type_id'] = "Ei aloitettu";
     	$palaute['Event']['count'] = $kaikkiasiakkaat - $count_not_started;
        if ($kaikkiasiakkaat == 0){
         	$palaute['Event']['percentage_all'] = 0;
         	$palaute['Event']['percentage_limit'] = 0;  
     	} else {
     	    $palaute['Event']['percentage_all'] = ($palaute['Event']['count'] / $kaikkiasiakkaat)*100;
     	    $palaute['Event']['percentage_limit'] = 0;  
     	}
     	$palauterivi[] = $palaute;

    // Luodaan palautusarray
    foreach ($tulos as $rivi ) {
     	$palaute['Event']['event_type_id'] = $rivi['event_types']['id'];
     	
     	// Tss pistetn left joinattujen tyhjien lukumr oikeaksi
     	if ($rivi['alakysely']['lkm'] == null){
     	   $palaute['Event']['count'] = 0;
     	} else {
     	  $palaute['Event']['count'] = $rivi['alakysely']['lkm'];  
     	}
     	
     	if ($kaikkiasiakkaat == 0){
         	$palaute['Event']['percentage_all'] = 0;
     	} else {
     	  $palaute['Event']['percentage_all'] = ($rivi['alakysely']['lkm'] / $kaikkiasiakkaat)*100;
     	}
     	
     	if ($asiakasrajauslkm == 0){
         	$palaute['Event']['percentage_limit'] = 0;  
     	} else {
          $palaute['Event']['percentage_limit'] = ($rivi['alakysely']['lkm'] / $asiakasrajauslkm)*100;
     	}

     	$palauterivi[] = $palaute;
    }

    return $palauterivi;

  }

  /**
   * Palauttaa sen kohdeasiakkaan, jolle seuraava puhelu tulee tehd pikasoittotoiminnallisuudessa. 
   * Virheellisen syteparametrin tai kannasta puuttuvan tunnistetiedon tapauksessa palauttaa false.
   * 
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan.
   * @param int $campaign_id Kampanjan tunnistetieto.
   * 
   * @return int Kohdeasiakkaan tunnistetieto, jolle soitto tehdn.
   */
  function chooseNextCall($user_id, $campaign_id){
    //tutkitaan, kuuluuko kyttj kampanjaan
    $oliko = $this->query("SELECT id FROM campaigns_users WHERE user_id = $user_id 
						AND campaign_id = $campaign_id",false);
    
   //jos ei, niin palautetaan false 
   if($oliko == null){
      return false;
    }
  
    //tarkistetaan ensin, onko rstiin jneit yhteydenottoja (ylittneet sovitun ajan tai ovat kohta ylittmss)
    $rastit = $this->query("SELECT target_client_id, contact_at FROM latest_events, target_clients WHERE user_id = $user_id
                           AND target_clients.campaign_id=$campaign_id AND target_clients.id=latest_events.target_client_id
                           AND target_clients.target_client_state_id NOT LIKE 'Tavoite saavutettu'
                           AND target_clients.target_client_state_id NOT LIKE 'Tavoitetta ei saavutettu'
                           AND (contact_at < now() 
                           OR (contact_at < addtime(now(),sec_to_time(".MYVA_CONST::TIME_FROM_PRESENT_TIME."))))  
                           ORDER BY contact_at ASC",false);
    
    //palautetaan mahd. yhteydenotto, joka on ollut kauimmin rstiss
    if($rastit != null){
      return $rastit['0']['latest_events']['target_client_id'];
    }
    
    //muuten katsotaan prioriteetin mukaan
    $tarkein = $this->query("SELECT contact_name, target_client_id, contact_at, priority FROM latest_events, event_types, target_clients 
						WHERE latest_events.user_id = $user_id AND target_clients.campaign_id=$campaign_id 
                        AND target_clients.id=latest_events.target_client_id
                        AND target_clients.target_client_state_id NOT LIKE 'Tavoite saavutettu'
                        AND target_clients.target_client_state_id NOT LIKE 'Tavoitetta ei saavutettu' 
                        AND latest_events.event_type_id = event_types.id
						ORDER BY priority DESC, contact_at ASC",false);

    if($tarkein!=null){
      //echo "palauttaa";
      //print_r($tarkein);
      return $tarkein[0]['events']['target_client_id'];
    }

    //kaikki tapahtumat, jos mitn ei lydy latest_events:t
    $tarkein = $this->query("SELECT target_client_id FROM events, target_clients 
						WHERE events.user_id = $user_id AND target_clients.campaign_id=$campaign_id 
                        AND target_clients.id=events.target_client_id",false);
    
    if ($tarkein!=null){
      return $tarkein['0']['events']['target_client_id'];
    }
    //null, jos ei ole asiakkaita
    else return null;
  }

  /**
   * Palauttaa raportin , joka sislt tiedot shkpostilinkkien klikkausten mrst.
   *
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan. Arvo -1 palauttaa
   * kaikkien kyttjien tiedot.
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param DateString $start_time = null Rajauksen aloitusaika.
   * @param DateString $end_time = null Rajauksen lopetusaika.
   * 
   * @return array Raportti-taulukko
   * <ul>
   *   <li>[0..n][MessageLink][target_url] string Linkin kohdeosoite.</li>
   *   <li>[0..n][MessageLink][target_url_text] string Linkin teksti.</li>
   *   <li>[0..n][EventLink][event_count] int Saatujen klikkausten lukumr.</li>
   *   <li>[0..n][EventLink][percentage_all] int Prosenttia kaikista.</li>
   *   <li>[0..n][EventLink][percentage_called] int Prosenttia lhetetyist shkposteista.</li>
   * </ul>
   */
  function getEmailLinksReport($user_id, $campaign_id, $start_time = null, $end_time = null){
    // Hakukyselyn aikarajausosio
    $aikarajaus = "";
    /** Kampanja id, joka pit hakea kyselyn kautta */
    $campaigns_user_id = 0;
    /** Kampanjaan kuuluvien kaikkien asiakkaiden lukumr */
    $kaikkiasiakkaat = 0;
    /** Asiakkaat, jotka kuuluvat rajaukseen */
    $asiakasrajauslkm = 0;
    //Kyttjn rajoitus kyselyiss
    $kayttajarajoitus = "";
    //Palaute
    $palauterivi = array();

    //Mahdollinen kyttjrajoitus
    if ($user_id <> -1){
      $campaigns_user_id = $this->getCampaignsUserId($user_id,$campaign_id);
      $kayttajarajoitus = "AND target_clients.campaigns_user_id = $campaigns_user_id";
    }
     
    //Mahdollinen aikarajaus
    if ($start_time !== null && $end_time === null ){
      $aikarajaus = "AND events.start_time >= '$start_time'";
    } else if ($start_time !== null && $end_time !== null){
      $aikarajaus = "AND events.start_time BETWEEN '$start_time' AND '$end_time'";
    }
    //Haetaan kaikkien asiakkaiden lukumr
    $kysely = $this->query("SELECT COUNT(*)
							FROM target_clients 
							WHERE campaign_id = $campaign_id
    						$kayttajarajoitus
							" );
    $kaikkiasiakkaat = $kysely[0][0]['COUNT(*)'];
   
    //Haetaan kohteet
    $kysely = $this->query("SELECT COUNT(*)
                           FROM target_clients, events 
						   WHERE target_clients.campaign_id = $campaign_id
                             AND target_clients.id IN 
								 (SELECT target_client_id FROM events 
								  WHERE (start_time) IN 
									    (SELECT max(start_time) 
									     FROM events as events2 
                                         WHERE events2.target_client_id = events.target_client_id and target_clients.campaign_id = $campaign_id
                                         AND event_type_id = 'Lhetetty'
                                         $aikarajaus 
								         $kayttajarajoitus
                                         GROUP BY target_client_id))
                             AND events.target_client_id = target_clients.id");
    $asiakasrajauslkm = $kysely[0][0]['COUNT(*)'];  
  
    //Haetaan kohteet
    $tulos = $this->query("SELECT target_url, target_url_text, SUM(events_message_links.event_count) AS lkm
                           FROM target_clients, events, events_message_links, message_links 
						   WHERE target_clients.campaign_id = $campaign_id
                             AND target_clients.id IN 
								 (SELECT target_client_id FROM events 
								  WHERE (target_client_id, start_time) IN 
									    (SELECT target_client_id, max(start_time) 
									     FROM events 
                                         WHERE target_clients.campaign_id = $campaign_id
                                         AND event_type_id = 'Klikattu'
                                         $aikarajaus 
								         $kayttajarajoitus
                                         GROUP BY target_client_id))
                             AND events.target_client_id = target_clients.id 
						     AND events_message_links.target_client_event_id = events.id 	
						     AND message_links.id = events_message_links.message_link_id 
						   GROUP BY message_links.target_url");

    // Luodaan palautusarray
    	foreach ($tulos as $rivi) {
       	    $palaute['MessageLink']['target_url'] = $rivi['message_links']['target_url'];
    		$palaute['MessageLink']['target_url_text'] = $rivi['message_links']['target_url_text'];
     		$palaute['EventLink']['event_count'] = $rivi[0]['lkm'];	
  		    $palaute['EventLink']['percentage_all'] = ($rivi[0]['lkm'] / $kaikkiasiakkaat)*100;
    		$palaute['EventLink']['percentage_called'] = ($rivi[0]['lkm'] / $asiakasrajauslkm)*100;
     		$palauterivi[] = $palaute;
    	}
       return $palauterivi;
  }

  /**
   * Palauttaa kampanjan tapahtumahistorian. Virheellisten parametrien tapauksessa tai kannasta puuttuvan
   * tunnistetiedon tapauksessa palauttaa false.
   * 
   * @param int $user_id Kyttjn tunnistetieto. Kuuluttava kampanjaan. Arvo -1 palauttaa
   * kaikkien kyttjien tiedot.
   * @param int $campaign_id Kampanjan tunnistetieto.
   * @param DateString $start_time = null Rajauksen aloitusaika.
   * @param DateString $end_time = null Rajauksen lopetusaika.
   * 
   * @return array Tapahtumahistoria-taulukko.
   * <ul>
   *   <li>[0..n][Event][id] int Tapahtuman tunnistetieto.</li>
   *   <li>[0..n][Event][start_time] string Tapahtuman aloitusaika.</li>
   *   <li>[0..n][Event][end_time] string Tapahtuman lopetusaika.</li>
   *   <li>[0..n][Event][contact_at] DateString Seuraavan yhteydenoton ajankohta. Arvo on null, jos ei ole mritelty.</li>
   *   <li>[0..n][Event][interest] int Kiinnostus.</li>
   *   <li>[0..n][Event][comment] string Kommentti.</li>
   *   <li>[0..n][Event][contact_name] string Yhteyshenkiln nimi.</li>
   *   <li>[0..n][Event][contact_title] string Yhteyshenkiln titteli.</li>
   *   <li>[0..n][Event][contact_phone_number] string Yhteyshenkiln puhelinnumero.</li>
   *   <li>[0..n][Event][contact_email_address] string Yhteyshenkiln shkpostiosoite.</li>
   *   <li>[0..n][Event][created] string Luontipiv.</li>
   *   <li>[0..n][Event][modified] string Viimeinen muutospiv.</li>
   *   <li>[0..n][Event][event_type_id] string Tapahtumatyypin nimi.</li>
   *   <li>[0..n][Event][target_client_id] string Kohdeasiakkaan tunnistetieto.</li>
   *   <li>[0..n][Event][user_id] string Kyttjn tunnistetieto.</li>
   *   <li>[0..n][Event][sales_argument_id] string Myyntiargumentin tunnistetieto.</li>
   *   <li>[0..n][Event][reason_not_interest_id] string Kiinnostumattomuuden syy.</li>
   *   <li>[0..n][User][name] string Kyttjn nimi.</li>
   *   <li>[0..n][TargetClient][name] string Kohdeasiakkaan nimi.</li>
   *   <li>[0..n][SalesArgument][argument] string Myyntiargumentti.</li>
   *   <li>[0..n][MessageLink][target_url] string Linkin kohdeosoite.</li>
   *   <li>[0..n][MessageLink][target_url_text] string Linkin teksti.</li>
   * </ul>
   */
  function getCampaignEvents($user_id, $campaign_id, $start_time = null, $end_time = null){
    //TODO testattava uudelleen kunhan on enemmn testikamaa
    //Hakuehtojen taulukko
    $hakuarray = null;
    //Hakuajan rajaus
    if ($start_time !== null && $end_time === null ){
      $hakuarray['Event.start_time'] = ">=$start_time";
    } else if ($start_time !== null && $end_time !== null){
      $hakuarray['Event.start_time'] = "between $start_time AND $end_time";
    }
    //Jos ei haeta kaikkia
    if ($user_id <> -1){
      $hakuarray['TargetClient.campaigns_user_id'] = $this->getCampaignsUserId($user_id, $campaign_id);
    }

    return $this->findAll($hakuarray);
  }

  
}
?>