<?php
/**
 * EventsMessageLinks
 * @author Ohjelmistotuotantoprojektiryhm myva, Helsingin yliopisto, Tietojenksittelytieteenlaitos
 * @version
 * @package myva
 * @subpackage Models
 */
class EventsMessageLink extends AppModel {

  var $name = 'EventsMessageLink';

  var $belongsTo = array('MessageLink' =>
  array('className'  => 'MessageLink'
                           ),
  'UserEvent' =>
  array('className'  => 'Event',
    'foreignKey' => 'user_event_id'
                           ),
  'TargetClientEvent' =>
  array('className'  => 'Event',
    'foreignKey' => 'target_client_event_id'
                           )
  );

  var $validate = array(
      'id' => VALID_SHA1,
      'event_count' => VALID_NON_NEG_INT,
      'message_link_id' => VALID_NON_NEG_INT,
      'user_event_id' => VALID_NON_NEG_INT,
      'target_client_event_id' => VALID_NON_NEG_INT_OR_EMPTY
      );
  
  
  /**
   * Prosessoi viestin avaamisesta saapuvan majakkakutsun tai linkin klikkauksen ja palauttaa URL-osoitteen, johon tehdn uudelleenohjaus.
   *
   * @param string $id Tapahtumalinkin tunniste (SHA1-merkkijono).
   * 
   * @return string URL-osoite, johon tehdn uudelleenohjaus tai null, jos tunniste oli virheellinen.
   */

	function messageEvent($id) {
	//* Parannettavaa: olisi jrkev, jos tietyn ajan sisll (esim. 30 sek) tulleita samoja tapahtumia ei kirjattaisi kuin kerran.
    $retries = MYVA_CONST::MAX_RETRIES;
    $success = false;
    
    if ($id === null || ! preg_match(VALID_SHA1, $id)) {
      return false;
    }

    while ($retries >= 0) {
      --$retries;

      $data = $this->find(array('EventsMessageLink.id' => $id), null, null, 1);
      if ($data === false) {
        return false;
      }

      $this->query('BEGIN');
      // Luodaan tapahtuma, jos sit ei viel ole olemassa.
      if ($data['EventsMessageLink']['target_client_event_id'] === null) {
        $data['TargetClientEvent'] = $data['UserEvent'];
        $data['TargetClientEvent']['id'] = null;
        unset($data['TargetClientEvent']['created']);
        if ($data['MessageLink']['beacon']) {
          $data['TargetClientEvent']['event_type_id'] = 'Avattu';
        } else {
          $data['TargetClientEvent']['event_type_id'] = 'Klikattu';
        }
      }
      $data['TargetClientEvent']['start_time'] = date('Y-m-d H:i:s');
      $data['TargetClientEvent']['end_time'] = $data['TargetClientEvent']['start_time'];
      unset($data['TargetClientEvent']['modified']);
      
      if (!$this->TargetClientEvent->save($data['TargetClientEvent'])) {
        $this->query('ROLLBACK');
        continue;
      }
      if ($data['EventsMessageLink']['target_client_event_id'] === null) {
        $data['TargetClientEvent']['id'] = $this->TargetClientEvent->getLastInsertID();
        $data['EventsMessageLink']['target_client_event_id'] = $data['TargetClientEvent']['id'];
      }

      $this->query('UPDATE events_message_links
        SET event_count = event_count + 1,
          target_client_event_id = ' . $data['EventsMessageLink']['target_client_event_id'] . '
        WHERE id = \'' . $data['EventsMessageLink']['id'] . '\'
        AND  event_count = ' . $data['EventsMessageLink']['event_count']);

      // Jos joku muu oli ehtinyt edelle, perutetaan kaikki ja yritetn uudelleen.
      $success = ($this->getAffectedRows() === 1);
      if ($success) {
        $this->query('COMMIT');
        break;
      } else {
        $this->query('ROLLBACK');
        continue;
      }
    }
    
    if ($success) {
      if ($data['MessageLink']['beacon']) {
        return MYVA_CONFIG::BEACON_URL;
      } else {
        return $data['MessageLink']['target_url'];
      }
    } else {
      return false;
    }
	}
	
}
?>