/**
 * Sivun latauduttua suoritettava funktio.
 * Alustaa SwapTable:t users_not_in_campaign ja users_in_campaign sek "klikkaa" niiden ensimmisi rivej.
 */
function onloadAction() {
  SwapTable.create('users_not_in_campaign', 'users_in_campaign', {orderBy:[1], visible:[false, true, true, true, true], afterSwap:[changeInputName, displayMovedUsersTargets]});
  SwapTable.create('users_in_campaign', 'users_not_in_campaign', {orderBy:[1], visible:[false, true, false, false, false], afterClick:displayClickedUsersTargets, beforeSwap:hasNotTargets, afterSwap:[changeInputName, userRemovedFromCampaign]});
  var rows = $('users_in_campaign').getElementsBySelector('tr');
  if (rows.length > 0) {
    $('users_in_campaign').clickRow(rows[0]);
  }
}


/**
 * Tarkistaa kampanjan tietojen lomakkeen ennen sen lhetyst.
 * Toistaiseksi ainoastaan palauttaa true.
 *
 * @return true 
 */
function checkCampaignForm(form) {
  return true;
}


/**
 * Luo uuden tyhjn input-elementin text_option_list -luokan elementtiin.
 *
 * @param {Element} currentField Elementti, jota aktivoitaessa tt funktiota kutsutaan.
 * @param {String} data_name Kentn sisltmn datan nimi, esim. "myyntiargumentti"
 * @param {int} num Juokseva numero - 1. Juokseva numero on yksilllinen jokaiselle input-kentlle
 *        text_option_list:n sisll. Jos currentField on viimeinen elementti, voi thn antaa sen
 *        numeron, sill tm funktio kasvattaa parametrina saamaansa arvoa yhdell.
 * @param {string} model Cake:n model, johon input-kentn tieto kuuluu.
 * @param {string} field Caken:n model:n kentt, johon input-kentn tieto tallennetaan.
 * @param {int} maxlen Input-kenttn kirjoitettavan tiedon maksimipituus.
 */
function newEmptyListField(currentField, data_name, num, model, field, maxlen) {
  if ($(currentField.parentNode).next() === undefined) {
    ++num;
    currentField.parentNode.parentNode.appendChild(Builder.node('div',{id:model + num}, [
      Builder.node('input', {
        className:'new',
        type:'text',
        name:'data[' + model + '][' + num + '][' + model + '][' + field + ']',
        value:'',
        title:'Uusi ' + data_name,
        maxlength:maxlen,
        onfocus:'newEmptyListField(this, \'' + data_name + '\', ' + num + ', \'' + model + '\', \'' + field + '\', ' + maxlen + ');',
        onblur:'textOptionListOnBlur(this, \'' + data_name + '\');'
      })
    ]));
  }
}


/**
 * Metodi, joka suoritetaan, kun kohdistus hvi TextOptionList:iin kuuluvasta input-elementist.
 * Metodi tekee seuraavat toimenpiteet:
 * <ul>
 *   <li>Asettaa input-kentn title-attribuutin arvoksi kentn sislln.
 *   <li>Poistaa mahdollisen ylimrisen uudelle sytteelle tarkoitetun input-elementin.
 *   <li>Jos input-elementin sislt ei ole poistettavissa, varoittaa kyttj ja palauttaa alkuperisen arvon.
 * </ul>
 */
function textOptionListOnBlur(inputField, data_name) {
  inputField = $(inputField);
  if (inputField.hasClassName('new') && isBlankString(inputField.value) && $(inputField.parentNode).next() !== undefined) {
    inputField.parentNode.remove();
  } else if (inputField.hasClassName('not_deletable') && isBlankString(inputField.value)) {
    inputField.value = inputField.oldValue;
    alert(data_name.charAt(0).toUpperCase() + data_name.substr(1) + ' "' + inputField.value + '" on jo kytss, eik sit voi poistaa.\nKentn sislt palautettiin alkuperiseksi.');
  } else if (isBlankString(inputField.value)) {
    inputField.title = 'Poistettu ' + data_name;
  } else {
    inputField.title = inputField.value;
  }
}


/**
 * Korvaa kaikkien parametrina annetuilla riveill olevien input-elementtien name-attribuutin arvoissa
 * swap_root:n id-attribuutin esiintymt kohde-SwapRoot:n id-attribuutin esiintymin.
 *
 * @param {Element} swap_root SwapRoot-elementti, jonka id:n esiintymt korvataan sen kohteen id:ll.
 * @param {Array{Element}} row Taulukko tr-elementeist, joiden input-elementtien name-attribuutit muutetaan.
 */
function changeInputName(swap_root, row) {
  for (var i = 0; i < row.length; ++i) {
    var input_elements = row[i].getElementsBySelector('input');
    for (var j = 0; j < input_elements.length; ++j) {
      input_elements[j].name = input_elements[j].name.replace(swap_root.id, swap_root.swapTarget.id);
    }
  }
  return true;
}


/**
 * Tuo nytlle nkyviin valittuna olevan kyttjn kohdeasiakkaat ja
 * piilottaa mahdollisesti edellisen valittuna olleen kyttjn kohdeasiakkaat.
 *
 * @param {Element} users_in_campaign_sr SwapTable, joka pit sislln kampanjaan kuuluvat kyttjt.
 */
function displayUsersTargets(users_in_campaign_sr) {
  var orphan_targets = $('orphan_targets');
  if (orphan_targets.swapTarget !== null) {
    $(orphan_targets.swapTarget).hide();
  }

  if (users_in_campaign_sr.selectedFirst != null) {
    var user_id = users_in_campaign_sr.selectedFirst.getElementsBySelector('.user_id');
    if (user_id.length > 0) {
      user_id = SwapTable.getElementText(user_id[0]);
    } else {
      return true;
    }
    var users_targets = $('user_' + user_id + '_targets');
    users_targets.show();
    orphan_targets.swapTarget = users_targets;
    $('targetsFromUser').onclick = function() {users_targets.swapSelected() };
  } else {
    orphan_targets.swapTarget = null;
  }
  return true;
}


/**
 * Tuo nkyviin (ensimmisen valitun) siirretyn kyttjn kohdeasiakkaat.
 *
 * @param {Element} swap_root SwapRoot, josta siirretn.
 @ @param {Array{Element}} row Siirrettvt rivit.
 */
function displayMovedUsersTargets(swap_root, row) {
  return displayUsersTargets(swap_root.swapTarget);
}


/**
 * Nytt klikatun kyttjn kohdeasiakkaat.
 *
 * @param {Element} swap_root SwapRoot, josta siirretn.
 @ @param {Event} e Klikkaustapahtuma.
 */
function displayClickedUsersTargets(clicked_element, e) {
  return displayUsersTargets(SwapTable.getSwapRoot(clicked_element));
}


/**
 * Tuo nkyviin ensimmisen valitun kyttjn kohdeasiakkaat, kun kyttj on poistettu kampanjasta.
 *
 * @param {Element} swap_root SwapRoot, josta siirretn.
 * @param {Array{Element}} row Siirrettvt rivit.
 */
function userRemovedFromCampaign(swap_root, row) {
  return displayUsersTargets(swap_root);
}


/**
 * Asettaa kohdeasiakkaan tietoihin muuttuneen/poistuneen kyttjn (myyjn).
 * Kytnnss luo kaksi input-kentt "changed_user"-luokan td-elementtiin
 * parametrina annetulla rivill, joista toinen sislt target_client_id:n ja
 * toinen user_id:n.
 *
 * @param {Element} swap_root SwapRoot, josta siirretn.
 * @param {Array{Element}} row Siirrettvt rivit.
 */
function setChangedUser(swap_root, row) {
  for (var i = 0; i < row.length; ++i) {
    var changed_user_cell = row[i].getElementsBySelector('.changed_user');
    if (changed_user_cell.length > 0) {
      changed_user_cell = changed_user_cell[0];
    } else {
      return true;
    }

    var target_client_id = SwapTable.getTextBySelector(row[i], '.target_client_id');
    if (target_client_id === null) {
      return true;
    }

    if (swap_root.swapTarget.id == 'orphan_targets') {
       var user_id = '';
    } else {
      var user_id = swap_root.swapTarget.id.replace('user_', '').replace('_targets', '');
    }

    // Jos elementtej ei viel ole, luodaan ne.
    // Muutosta tlle kohdeasiakkaalle ei siin tapauksessa ole viel tehty.
    var target_client_id_input = $('TargetClient' + target_client_id + 'TargetClientId');
    if (target_client_id_input === null) {
      target_client_id_input = Builder.node('input', {
          className:'id',
          type:'hidden',
          name:'data[TargetClient][' + target_client_id + '][TargetClient][id]',
          value:target_client_id,
          id:'TargetClient' + target_client_id + 'TargetClientId'
        });
      changed_user_cell.appendChild(target_client_id_input);
    }
    var user_id_input = $('TargetClient' + target_client_id + 'TargetClientUserId');
    if (user_id_input === null) {
      user_id_input = Builder.node('input', {
          className:'id',
          type:'hidden',
          name:'data[TargetClient][' + target_client_id + '][TargetClient][user_id]',
          value:'',
          id:'TargetClient' + target_client_id + 'TargetClientUserId'
        });
      changed_user_cell.appendChild(user_id_input);
    }

    // Asetetaan uusi user_id.
    user_id_input.value = user_id;
  }

  return true;
}


/**
 * Tarkistaa, ettei siirrettvill myyjill ole asiakkaita.
 * Myyjn asiakkaat ovat SwapTable-elementiss, jonka id on "user_USERID_targets",
 * miss USERID on kyttjn tunniste.
 *
 * @param {Element} swap_root SwapRoot, josta siirretn.
 * @param {Array{Element}} row Siirrettvt rivit.
 *
 * @return true, jos myyjll ei ole asiakkaita, muutoin false.
 */
function hasNotTargets(swap_root, row) {
  for (var i = 0; i < row.length; ++i) {
    var user_id = SwapTable.getTextBySelector(row[i], '.user_id');
    var user_targets = $('user_' + user_id + '_targets').getElementsBySelector('tr');
    if (user_targets.length > 0) {
      alert('Myyjll ' + SwapTable.getTextBySelector(row[i], '.user_name') + ' on ' + user_targets.length + ' asiakasta.\nPoistaaksesi myyjn kampanjasta siirr ensin myyjn asiakkaat pois myyjlt.');
      return false;
    }
  }
  return true;
}
